from textstat.textstat import textstat
from multilang_summarizer.spanish_syllabicator import Silabicador

def flesch_kincaid(tokenized_sentences):
    total_syllables = 0
    total_words = 0
    total_sentences = len(tokenized_sentences)
    for tokenized_sent in tokenized_sentences:
        for token in tokenized_sent:
            total_words += 1
            total_syllables += textstat.syllable_count(token)

    score = 206.835
    score -= 1.015 * (total_words / total_sentences)
    score -= 84.6 * (total_syllables / total_words)
    return score / 100

def szigriszt_pazos(tokenized_sentences):
    syl = Silabicador()
    total_syllables = 0
    total_words = 0
    total_sentences = len(tokenized_sentences)
    for tokenized_sent in tokenized_sentences:
        for token in tokenized_sent:
            total_words += 1
            total_syllables += len(syl(token)[0])
    score = 206.835
    score -= 62.3 * (total_syllables / total_words)
    score -= total_words / total_sentences
    return score / 100

def invert(method, tokenized_sentences):
    return 1 - method(tokenized_sentences)



from math import log
import numpy as np
import sys

sys.setrecursionlimit(10000)

def flatten(multidim_list):
    if isinstance(multidim_list, list):
        if len(multidim_list) > 1:
            return flatten(multidim_list[0]) + flatten(multidim_list[1:])
        elif len(multidim_list) > 0:
            return flatten(multidim_list[0])
        else:
            return []
    else:
        return [multidim_list]

def calculate_idf(tokenized_documents):
    type_set = list(set(flatten(tokenized_documents)))
    idf = {}
    total_num_of_docs = len(tokenized_documents)
    for word in type_set:
        idf[word] = 0
        for document in tokenized_documents:
            if word in document:
                idf[word] += 1
        idf[word] = log(total_num_of_docs / idf[word])
    return idf

def calculate_tf(word, tokenized_document):
    return tokenized_document.count(word) / len(tokenized_document)

def tfidf_matrix(tokenized_documents):
    idf = calculate_idf(tokenized_documents)
    matrix = []
    terms = sorted(list(idf.keys()))
    for document in tokenized_documents:
        doc_vector = []
        for word in terms:
            doc_vector.append(calculate_tf(word, document) * idf[word])
        matrix.append(doc_vector)
    return matrix, terms, idf

def represent_in_tfidf(tokenized_document, terms, idf):
    new_vector = []
    for term in terms:
        new_vector.append(calculate_tf(term, tokenized_document) * idf[term])
    return np.array(new_vector)




import sys

sys.setrecursionlimit(2000)

def lcs_matrix(s1, s2):
    matrix=[[0 for _ in range(len(s2) + 1)] for _ in range(len(s1) + 1)]
    for i in range(len(s1) + 1):
        for j in range(len(s2) + 1):
            if i == 0 or j == 0:
                    matrix[i][j] = 0
            elif s1[i-1] == s2[j-1]:
                matrix[i][j] = matrix[i-1][j-1] + 1
            else:
                matrix[i][j] = max(matrix[i-1][j], matrix[i][j-1])
    return matrix

def indexes_backtrack_lcs_matrix(matrix, s1, s2, i, j):
    if i == 0 or j == 0:
        return []

    if s1[i-1] == s2[j-1]:
        return indexes_backtrack_lcs_matrix(matrix, s1, s2, i-1, j-1) + [(i-1, j-1)]
    if matrix[i][j-1] > matrix[i-1][j]:
        return indexes_backtrack_lcs_matrix(matrix, s1, s2, i, j-1)
    return indexes_backtrack_lcs_matrix(matrix, s1, s2, i-1, j)

def backtrack_lcs_matrix(matrix, s1, s2, i, j):
    if i == 0 or j == 0:
        return []

    if s1[i-1] == s2[j-1]:
        return backtrack_lcs_matrix(matrix, s1, s2, i-1, j-1) + [s1[i-1]]
    if matrix[i][j-1] > matrix[i-1][j]:
        return backtrack_lcs_matrix(matrix, s1, s2, i, j-1)
    return backtrack_lcs_matrix(matrix, s1, s2, i-1, j)

def all_backtrack_lcs_matrix(matrix, s1, s2, i, j):
    if i == 0 or j == 0:
        return [[]]
    elif s1[i-1] == s2[j-1]:
        return [s + [s1[i-1]] for s in all_backtrack_lcs_matrix(matrix,
                                                                s1, s2,
                                                                i-1, j-1)]
    else:
        result = []
        if matrix[i][j-1] >= matrix[i-1][j]:
            result = result + all_backtrack_lcs_matrix(matrix,
                                                       s1, s2,
                                                       i, j-1)
        if matrix[i-1][j] >= matrix[i][j-1]:
            result = result + all_backtrack_lcs_matrix(matrix,
                                                       s1, s2,
                                                       i-1, j)
        return result

def lcs(s1, s2, joiner=""):
    matrix = lcs_matrix(s1, s2)
    if joiner ==  "":
        return backtrack_lcs_matrix(matrix, s1, s2, len(s1), len(s2)) 
    return joiner.join(backtrack_lcs_matrix(matrix, s1, s2, len(s1), len(s2)))

def lcs_indexes(s1, s2, joiner=""):
    matrix = lcs_matrix(s1, s2)
    if joiner ==  "":
        return indexes_backtrack_lcs_matrix(matrix, s1, s2, len(s1), len(s2)) 
    return joiner.join(indexes_backtrack_lcs_matrix(matrix, s1, s2, len(s1), len(s2)))

def all_lcs(s1, s2, joiner=""):
    matrix = lcs_matrix(s1, s2)
    result = []
    for elem in all_backtrack_lcs_matrix(matrix, s1, s2, len(s1), len(s2)):
        result.append(joiner.join(elem))
    return list(set(result))



import pathlib

import copy
import os
import sys

import pickle

from collections import Counter

from functools import partial

from math import log

from multilang_summarizer.lcs import *

from multilang_summarizer.tfidf import calculate_tf, calculate_idf
from multilang_summarizer.entropy import syllable_metric_entropy

PARENT_DIR = pathlib.Path(__file__).parent

def get_likelihoods(lookup_table):
    likelihoods = {}
    terms = list(lookup_table.keys())
    values = list(lookup_table.values())
    total_seen = sum(values)
    for term in terms:
        likelihoods[term] = lookup_table[term] / total_seen
    return likelihoods

def overlapping_tokens(tokens1, tokens2):
    '''fake lcs from overlapping tokens'''
    all_tokens = []
    all_token_indexes = []
    modif_tokens1 = tokens1.copy()
    modif_tokens2 = tokens2.copy()
    for i, token in enumerate(modif_tokens1):
        if token in modif_tokens2:
            j = modif_tokens2.index(token)
            modif_tokens2[j] = " "
            all_tokens.append(token)
            all_token_indexes.append((i, j))
    return all_tokens, all_token_indexes

def probabilistic_tokens(tokens1, tokens2):
    '''fake lcs from overlapping tokens'''
    all_tokens = []
    all_token_indexes = []

    probabilities = []

    modif_tokens1 = tokens1.copy()
    modif_tokens2 = tokens2.copy()

    token_counter = Counter(modif_tokens1 + modif_tokens2)
    total = len(modif_tokens1) + len(modif_tokens2)

    for i, token in enumerate(modif_tokens1):
        if token in modif_tokens2:
            j = modif_tokens2.index(token)
            modif_tokens2[j] = " "
            all_tokens.append(token)
            all_token_indexes.append((i, j))
            probabilities.append(token_counter[token] / total)

    probabilities = list(enumerate(probabilities))
    probabilities.sort(key=lambda x: x[1], reverse=True)
    max_tokens = len(lcs(tokens1, tokens2))
    count = 0
    for i, prob in probabilities:
        if count < max_tokens:
            all_tokens[i] == None
            all_token_indexes[i] == None
            count += 1
    all_tokens = [t for t in all_tokens if t != None]
    all_token_indexes = [t for t in all_token_indexes if t != None]
    return all_tokens, all_token_indexes

def lcs_tokens(tokens1, tokens2):
    return lcs(tokens1, tokens2), lcs_indexes(tokens1, tokens2)

def create_index(aligned_sentences):
    index = {}
    start = 0
    sentence_number = 0
    for sent, tok_sent, lem_sent in aligned_sentences:
        for i in range(start, start + len(lem_sent)):
            index[i] = (sentence_number, sent, tok_sent, lem_sent)
        start += len(lem_sent)
        sentence_number += 1
    return index

class Document(object):

    def __init__(self, path, lemmatizer, remove_stopwords=True):
        self._lemmatizer = lemmatizer
        self._syllabicator = lemmatizer.syllabicator
        with open(path, "r") as f:
            self.raw_text = f.read()
        self.sentences = lemmatizer.lemmatize(self.raw_text,
                                              remove_stopwords=remove_stopwords)
        self.raw_sentences = self.sentences[0]
        self.tok_sentences = self.sentences[1]
        self.lem_sentences = self.sentences[2]

        self.tok_text = []
        for sentence in self.tok_sentences:
            for token in sentence:
                self.tok_text.append(token)

        self.lem_text = []
        for sentence in self.lem_sentences:
            for token in sentence:
                self.lem_text.append(token)

        # align sentences
        self.aligned_sentences = []
        for i in range(len(self.raw_sentences)):
            self.aligned_sentences.append((self.raw_sentences[i],
                                           self.tok_sentences[i],
                                           self.lem_sentences[i]))


class F1(object):

    def __call__(self,
                 relevant_term_sequence,
                 tokenized_sentence,
                 likelihoods,
                 syllabicator):
        '''sum of rel term likelihoods / number of syls
        '''
        current_score = 0.
        for term in relevant_term_sequence:
            current_score += likelihoods[term]
        # total syllables
        total_syllables = 0.
        for term in tokenized_sentence:
            total_syllables += len(syllabicator(term))
        return current_score / total_syllables

class F2(object):

    def __call__(self,
                 relevant_term_sequence,
                 tokenized_sentence,
                 relevance_table,
                 syllabicator):
        '''(sum of rel term lklihoods * lambda / number of terms) - syllable entropy
        '''
        current_score = 0.
        for term in relevant_term_sequence:
            current_score += relevance_table[term]

        current_score /= len(tokenized_sentence)
        syllable_entropy = syllable_metric_entropy(tokenized_sentence, syllabicator)
        return current_score - syllable_entropy

class F3(object):

    def __call__(self,
                 relevant_term_sequence,
                 tokenized_sentence,
                 lemmatized_sentence,
                 relevance_table,
                 idf,
                 syllabicator):
        '''(sum of rel term tfidf scores * lambda) / number of syllables
        '''
        current_score = 0.
        for term in relevant_term_sequence:
            tfidf_score = calculate_tf(term, lemmatized_sentence) * idf[term]
            current_score += tfidf_score * relevance_table[term]
        # total syllables
        total_syllables = 0.
        for term in tokenized_sentence:
            total_syllables += len(syllabicator(term))
        return current_score / total_syllables

def clean_working_memory():
    try:
        for f_path in os.listdir(PARENT_DIR / "data/temp/"):
            os.remove(PARENT_DIR / "data/temp/" + f_path)
    except:
        pass

def summarizer(D_path, f_method, seq_method, lemmatizer, session_id=1):

    # Try creating working memory
    try:
        os.makedirs(PARENT_DIR / "data/temp/")
    except:
        pass

    if f_method == "f1":
        f = F1()
    elif f_method == "f2":
        f = F2()
    elif f_method == "f3":
        f = F3()
    else:
        raise Exception("Invalid relevance function")

    if seq_method == "partial":
        seq = overlapping_tokens
    elif seq_method == "probabilistic":
        seq = probabilistic_tokens
    elif seq_method == "lcs":
        seq = lcs_tokens
    else:
        raise Exception("Invalid relevance term sequence calculation method")

    D = Document(D_path, lemmatizer)

    if os.path.exists(PARENT_DIR / ("data/temp/running_summary_%d.pickle" % session_id)):
        with open(PARENT_DIR / ("data/temp/running_summary_%d.pickle" % session_id), "rb") as fp:
            RS = pickle.load(fp)
    else:
        RS = None

    if os.path.exists(PARENT_DIR / ("data/temp/lookup_table_%d.pickle" % session_id)):
        with open(PARENT_DIR / ("data/temp/lookup_table_%d.pickle" % session_id), "rb") as fp:
            lookup_table = pickle.load(fp)
    else:
        lookup_table = {}

    if RS is None:
        RS = D
        new_RS_sent_scores = None
    else:
        # Calculate idf for tfidf, preemptively
        idf = calculate_idf(RS.lem_sentences + D.lem_sentences)

        # Algorithm starts properly
        sequence_of_rt, index_pairs = seq(RS.lem_text,
                                          D.lem_text)

        index_to_sent_in_RS = create_index(RS.aligned_sentences)
        index_to_sent_in_D = create_index(D.aligned_sentences)

        new_RS = []
        new_RS_sent_scores = []

        for term in sequence_of_rt:
            # Log terms
            lookup_table[term] = lookup_table.get(term, 0) + 1
        likelihoods = get_likelihoods(lookup_table)

        while len(sequence_of_rt) > 0:
            term = sequence_of_rt.pop(0)
            term_index_in_RS, term_index_in_D = index_pairs.pop(0)

            # Recover sentences
            candidate_in_RS = index_to_sent_in_RS[term_index_in_RS]
            candidate_in_D = index_to_sent_in_D[term_index_in_D]

            if f_method == "f1":
                score_for_RS_candidate = f(sequence_of_rt,
                                           candidate_in_RS[2],
                                           likelihoods,
                                           RS._syllabicator
                                          )
                score_for_D_candidate = f(sequence_of_rt,
                                          candidate_in_D[2],
                                          likelihoods,
                                          D._syllabicator
                                         )
            elif f_method == "f2":
                score_for_RS_candidate = f(sequence_of_rt,
                                           candidate_in_RS[2],
                                           lookup_table,
                                           RS._syllabicator
                                          )
                score_for_D_candidate = f(sequence_of_rt,
                                          candidate_in_D[2],
                                          lookup_table,
                                          D._syllabicator
                                         )
            elif f_method == "f3":
                score_for_RS_candidate = f(sequence_of_rt,
                                           candidate_in_RS[2],
                                           candidate_in_RS[3],
                                           lookup_table,
                                           idf,
                                           RS._syllabicator
                                          )
                score_for_D_candidate = f(sequence_of_rt,
                                          candidate_in_D[2],
                                          candidate_in_D[3],
                                          lookup_table,
                                          idf,
                                          D._syllabicator
                                         )

            if score_for_RS_candidate > score_for_D_candidate:
                new_RS.append(candidate_in_RS)
                new_RS_sent_scores.append(score_for_RS_candidate)
                # remove other terms from the sequence if included in the
                # candidate
                for_is_finished = False
                breaked = False
                while not for_is_finished:
                    for i in range(len(sequence_of_rt)):
                        future_term_index_in_RS = index_pairs[i][0]
                        future_candidate_in_RS =\
                            index_to_sent_in_RS[future_term_index_in_RS]
                        if future_candidate_in_RS[0] == candidate_in_RS[0]:
                            # Same sentence --> remove
                            del sequence_of_rt[i]
                            del index_pairs[i]
                            breaked = True
                            break
                    if not breaked:
                        for_is_finished = True
                    else:
                        breaked = False
            else:
                new_RS.append(candidate_in_D)
                new_RS_sent_scores.append(score_for_D_candidate)
                # remove other terms from the sequence if included in the
                # candidate
                for_is_finished = False
                breaked = False
                while not for_is_finished:
                    for i in range(len(sequence_of_rt)):
                        future_term_index_in_D = index_pairs[i][1]
                        future_candidate_in_D =\
                            index_to_sent_in_D[future_term_index_in_D]
                        if future_candidate_in_D[0] == candidate_in_D[0]:
                            # Same sentence --> remove
                            del sequence_of_rt[i]
                            del index_pairs[i]
                            breaked = True
                            break
                    if not breaked:
                        for_is_finished = True
                    else:
                        breaked = False

        new_RS_raw_sentences = []
        for _, raw_sentence, _, _ in new_RS:
            new_RS_raw_sentences.append(raw_sentence)

        # Replace old RS
        with open(PARENT_DIR / ("data/temp/running_summary_%d.txt" % session_id), "w") as fp:
            fp.write("\n".join(new_RS_raw_sentences))
        RS = Document(PARENT_DIR / ("data/temp/running_summary_%d.txt" % session_id), lemmatizer)

    with open(PARENT_DIR / ("data/temp/running_summary_%d.pickle" % session_id), "wb") as fp:
        pickle.dump(RS, fp, protocol=pickle.HIGHEST_PROTOCOL)

    with open(PARENT_DIR / ("data/temp/lookup_table_%d.pickle" % session_id), "wb") as fp:
        pickle.dump(lookup_table, fp, protocol=pickle.HIGHEST_PROTOCOL)

    return RS, new_RS_sent_scores

def summary_limit(summary_sentences, sentence_scores, byte_limit):
    limited_summary = []
    current_length = 0
    score_position = [(i,score) for i, score in enumerate(sentence_scores)]
    score_position.sort(key=lambda x:x[1], reverse=True)

    for position, score in score_position:
        raw_sent, _, _ = summary_sentences[position]
        if (current_length + len(raw_sent.encode("utf-8"))) <= byte_limit:
            limited_summary.append(position)
            current_length += len(raw_sent.encode("utf-8"))
    limited_summary.sort()
    limited_summary = [summary_sentences[pos] for pos in limited_summary]
    return limited_summary

def summary_wordlimit(summary_sentences, sentence_scores, word_limit):
    limited_summary = []
    current_length = 0
    score_position = [(i,score) for i, score in enumerate(sentence_scores)]
    score_position.sort(key=lambda x:x[1], reverse=True)

    for position, score in score_position:
        raw_sent, _, _ = summary_sentences[position]
        if (current_length + raw_sent.count(" ") + 1) <= word_limit:
            limited_summary.append(position)
            current_length += raw_sent.count(" ") + 1
    limited_summary.sort()
    limited_summary = [summary_sentences[pos] for pos in limited_summary]
    return limited_summary



# -*- coding:utf-8 -*-

import copy

class CharArray(object):
    
    def __init__(self, word):
        if not isinstance(word, str):
            raise TypeError("No the correct type for a char array")
        
        self.vocales_fuertes = ["a", "e", "o", "á", "é", "ó", "í", "ú", "ä"]
        self.vocales_debiles = ["i", "u"]
        self.vocales = self.vocales_debiles + self.vocales_fuertes

        self.consonant_y = ["y" + vowel for vowel in self.vocales]
        
        diptongos_crecientes = [d + f for d in self.vocales_debiles\
                                      for f in self.vocales_fuertes]
        diptongos_decrecientes = [f + d for d in self.vocales_debiles\
                                        for f in self.vocales_fuertes] + \
                                 [f + "y" for f in self.vocales_fuertes]
        diptongos_homogeneos = ["iu", "ui"]
        self.diptongos = diptongos_crecientes + \
                         diptongos_decrecientes + \
                         diptongos_homogeneos
        
        self.triptongos = [dip + d for dip in diptongos_crecientes \
                                   for d in self.vocales_debiles] +\
                          [dip + "y" for dip in diptongos_crecientes if \
                                             not dip.endswith("y")]
                
        self.grupos_inseparables = ["br", "cr","dr", "gr", "fr", "kr", "tr", "bl", 
                                    "cl", "gl", "fl", "kl", "pl", "tl", "ll", "ch",
                                    "rr", "pr", "qu", "gü"]
        
        self.word = word
        self.vocal_representation, self.mask_lookup = \
                                        self.build_abstract_representation(word)
        
    def build_abstract_representation(self, word):

        mask_lookup = []

        if word == "y":
            word = "|"
            mask_lookup.append(("V", 0, "y"))

        for consonant_y in self.consonant_y:
            while consonant_y in word:
                index_to_replace = word.index("y")
                word = word.replace("y", "#", 1)
                mask_lookup.append(("C", [index_to_replace,
                                          index_to_replace + 1], "y"))

        for triptongo in self.triptongos:
            while triptongo in word:
                index_to_replace = word.index(triptongo)
                word = word.replace(triptongo, "$$$", 1)
                mask_lookup.append(("V", [index_to_replace,
                                          index_to_replace + 3],
                                    triptongo))

        for diptongo in self.diptongos:
            while diptongo in word:
                index_to_replace = word.index(diptongo)
                word = word.replace(diptongo, "@@", 1)
                mask_lookup.append(("V", [index_to_replace,
                                         index_to_replace + 2],
                                    diptongo))

        for grupo_c in self.grupos_inseparables:
            while grupo_c in word:
                index_to_replace = word.index(grupo_c)
                word = word.replace(grupo_c, "&&", 1)
                mask_lookup.append(("C", [index_to_replace,
                                          index_to_replace + 2],
                                    grupo_c))

        for vowel in self.vocales_debiles + ["y"]:
            while vowel in word:
                index_to_replace = word.index(vowel)
                word = word.replace(vowel, "|", 1)
                mask_lookup.append(("V", [index_to_replace,
                                          index_to_replace + 1], vowel))

        for vowel in self.vocales_fuertes:
            while vowel in word:
                index_to_replace = word.index(vowel)
                word = word.replace(vowel, "¬", 1)
                mask_lookup.append(("V", [index_to_replace,
                                         index_to_replace + 1], vowel))

        for consonant in list("bcdfghjklmnñpqrstvwxz"):
            while consonant in word:
                index_to_replace = word.index(consonant)
                word = word.replace(consonant, "[", 1)
                mask_lookup.append(("C", [index_to_replace,
                                          index_to_replace + 1], 
                                    consonant))
 
        extra_chars = word.replace("|", "").replace("#", "").replace("$", "")\
                .replace("@", "").replace("&", "").replace("¬", "")\
                .replace("[", "")

        if extra_chars != "":
            for extra_char in list(extra_chars):
                while extra_char in word:
                    index_to_replace = word.index(extra_char)
                    word = word.replace(extra_char, "]", 1)
                    mask_lookup.append(("C", [index_to_replace,
                                              index_to_replace + 1],
                                        extra_char))
        #   | -> y_vocal, vocal debil
        #   # -> y_consonante
        # $$$ -> triptongo
        #  @@ -> diptongo
        #  && -> grupo consonantico
        #  ¬ -> vocal fuerte
        #  [ -> consonante
        #  ] -> otros caracteres (considerados consonante)
        word = word.replace("|", "V").replace("#", "C").replace("$$$", "V")\
                .replace("@@", "V").replace("&&", "C").replace("¬", "V")\
                .replace("[", "C").replace("]", "C")
        mask_lookup = sorted(mask_lookup, key=lambda elem: elem[1])
        return word, mask_lookup

    def unmask(self, pattern):
        result = []
        syllabic_pattern = []
        mask_lookup = copy.copy(self.mask_lookup)
        for syllable in pattern:
            subsyl = ""
            subsylpattern = ""
            for character in syllable:
                remembered_char, interval, corresponding_string = \
                                                            mask_lookup.pop(0)
                if character == remembered_char:
                    subsyl += corresponding_string
                    subsylpattern += remembered_char * len(corresponding_string)
                else:
                    raise ValueError("The pattern doesn't correspond to this\
                                     word: %s" % self.word)
            result.append(subsyl)
            syllabic_pattern.append(subsylpattern)
        return result, syllabic_pattern

    def __str__(self, *args, **kwargs):
        return str(self.vocal_representation)
    
    def __repr__(self, *args, **kwargs):
        return str(self)
    
    
class Silabicador(object):
    
    def __call__(self, word):
        '''http://ponce.inter.edu/acad/cursos/ciencia/lasvi/modulo2.htm'''
        res = []
        lower_word = word.lower()
        char_array = CharArray(lower_word)
        abstract_word = list(str(char_array))
        while len(abstract_word) != 0:
            if abstract_word[0] == "V":
                if len(abstract_word) == 1:
                    res += ["V"]
                    abstract_word = []
                elif len(abstract_word) == 2:
                    if abstract_word[1] == "C":
                        res += ["VC"]
                    else:
                        res += ["V", "V"]
                    abstract_word = []
                elif len(abstract_word) == 3:
                    res += ["V", abstract_word[1] + abstract_word[2]]
                    abstract_word = []
                else:
                    # No hay consonantes en frente, sino otra vocal
                    if abstract_word[1] == "V":
                        res += ["V"]
                        del abstract_word[0]
                    # Una consonante entre dos vocales se agrupa con la vocal de la derecha:
                    elif abstract_word[1] == "C" and\
                       abstract_word[2] == "V":
                        res += ["V", "CV"]
                        del abstract_word[2]
                        del abstract_word[1]
                        del abstract_word[0]
                    # Dos consonantes entre dos vocales se separan y cada consonante se queda con una vocal:
                    elif abstract_word[1] == "C" and\
                       abstract_word[2] == "C" and\
                       abstract_word[3] == "V":
                        res += ["VC", "CV"]
                        del abstract_word[3]
                        del abstract_word[2]
                        del abstract_word[1]
                        del abstract_word[0]

                    # Cuando hay tres consonantes entre vocales, las primeras dos se unen con la primera vocal y la tercera se une a la segunda vocal.
                    elif len(abstract_word) > 4 and\
                         abstract_word[1] == "C" and\
                         abstract_word[2] == "C" and\
                         abstract_word[3] == "C" and\
                         abstract_word[4] == "V":
                        res += ["VCC", "CV"]
                        del abstract_word[4]
                        del abstract_word[3]
                        del abstract_word[2]
                        del abstract_word[1]
                        del abstract_word[0]
                    # Cuando hay cuatro consonantes entre vocales, las primeras dos se unen a la primera vocal y las otras dos se unen a la segunda vocal.
                    elif len(abstract_word) > 5 and\
                         abstract_word[1] == "C" and\
                         abstract_word[2] == "C" and\
                         abstract_word[3] == "C" and\
                         abstract_word[4] == "C" and\
                         abstract_word[5] == "V":
                        res += ["VCC", "CCV"]
                        del abstract_word[5]
                        del abstract_word[4]
                        del abstract_word[3]
                        del abstract_word[2]
                        del abstract_word[1]
                        del abstract_word[0]
                    # Chain of consonants
                    else:
                        count = 0
                        index = 1
                        final_letter = ""
                        while index < len(abstract_word) and abstract_word[index] != "V":
                            final_letter = abstract_word[index]
                            index += 1
                            count += 1
                        if index == len(abstract_word):
                            index -= 1
                        if final_letter == "V":
                            res += ["VCC", "C"*(count-4), "CCV"]
                        else:
                            res += ["VCC", "C"*(count-2)]
                        for i in range(index, -1, -1):
                            del abstract_word[i]
            
            elif abstract_word[0] == "C":
                res.append(abstract_word.pop(0))
        
        final_grouping = []
        while len(res)>0:
            if res[0] == "C" and \
               len(res) != 1 and \
               res[1].startswith("V"):
                final_grouping.append(res[0] + res[1])
                del res[1]
                del res[0]
            # Si existe la consonante pega con la silaba anterior
            elif res[0] == "C" and\
                 len(final_grouping)>0 and\
                 final_grouping[-1].endswith("V"): 
                final_grouping[-1] = final_grouping[-1] + res[0]
                del res[0]
            # Else, assume it is a valid syllable
            else:
                final_grouping.append(res[0])
                del res[0]

        return char_array.unmask(final_grouping)



from math import log
from statistics import mean, stdev, variance
from collections import Counter

def shannon_entropy(group):
    total_members = len(group)
    entropy = 0

    frequencies = Counter(group)
    for unique_class in frequencies:
        likelihood = frequencies[unique_class] / total_members
        entropy += likelihood * log(likelihood, 2)

    return (-1) * entropy

def normalized_shannon_entropy(group):
    return shannon_entropy(group) / len(group)

def kullback_leibler_divergence(group_x, group_y):
    # - sum_i p(i)*log( q(i)/p(i) ,2)
    entropy = 0

    likelihood_x = {}
    likelihood_y = {}

    frequencies_x = Counter(group_x)
    total_x = len(group_x) + len(set(group_x))
    for unique_class in frequencies_x:
        likelihood_x[unique_class] = (frequencies_x[unique_class] + 1) / total_x

    frequencies_y = Counter(group_y)
    total_y = len(group_y) + len(set(group_y))
    for unique_class in frequencies_y:
        likelihood_y[unique_class] = (frequencies_y[unique_class] + 1) / total_y

    common_space = set(group_x + group_y)
    for member in common_space:
        p_i = likelihood_x.get(member, 1/total_x)
        q_i = likelihood_y.get(member, 1/total_y)
        entropy += p_i * log(q_i/p_i, 2)
    return (-1) * entropy

def joint_shannon_entropy(group_x, group_y):
    # sum_xy p(x,y)*log( p(x)/p(x,y) ,2)

    entropy = 0

    likelihood_x = {}
    likelihood_y = {}

    frequencies_x = Counter(group_x)
    for unique_class in frequencies_x:
        likelihood_x[unique_class] = frequencies_x[unique_class] / len(group_x)

    frequencies_y = Counter(group_y)
    for unique_class in frequencies_y:
        likelihood_y[unique_class] = frequencies_y[unique_class] / len(group_y)

    for x in likelihood_x:
        p_x = likelihood_x[x]
        for y in likelihood_y:
            p_x_y = p_x * likelihood_y[y]
            entropy += p_x_y * log(p_x_y, 2)
    return (-1) * entropy

def conditional_entropy(group_y, group_x):
    # H(Y|X) = H(X, Y) - H(X)
    joint_entropy = joint_shannon_entropy(group_x, group_y)
    x_entropy = shannon_entropy(group_x)
    return joint_entropy - x_entropy

def mutual_information(group_x, group_y):
    # I(X;Y) = H(X) + H(Y) - H(X, Y)
    x_entropy = shannon_entropy(group_x)
    y_entropy = shannon_entropy(group_y)
    joint_entropy = joint_shannon_entropy(group_x, group_y)
    return x_entropy + y_entropy - joint_entropy

def information_gain(group_x, group_y):
    # IG(X;Y) = H(X) - H(X|Y)
    x_entropy = shannon_entropy(group_x)
    x_given_y_entropy = conditional_entropy(group_x, group_y)
    return x_entropy - x_given_y_entropy

def relevant_term_probabilities(relevant_terms):
    counts = Counter(relevant_terms)
    probabilities = {}
    for term in counts:
        probabilities[term] = counts[term] / len(relevant_terms)
    return probabilities

def syllable_metric_entropy(sentence, syllabicator):
    syllables = []
    for token in sentence:
        token_syllables = syllabicator(token)
        if len(token_syllables) == 0:
            token_syllables = [token]
        syllables += token_syllables

    try:
        syllable_entropy = metric_shannon_entropy(syllables)
    except:
        syllable_entropy = 1.0

    return syllable_entropy




import pathlib
import os

from multilang_summarizer.lemmatizer import Lemmatizer, language_index, nltk_stopwords
from multilang_summarizer.summarizer import summarizer

available_languages = []
for key in language_index:
    if key in nltk_stopwords.keys():
        available_languages.append(key)



import pathlib
from setuptools import setup

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

setup(
    name = 'multilang-summarizer',
    packages = ['multilang_summarizer'],
    version = '1.7',
    license='GPLv3',
    description = 'Multilanguage summarizer, intended to improve text readability',
    long_description = README,
    long_description_content_type = "text/markdown",
    author = 'Arturo Curiel',
    author_email = 'me@arturocuriel.com',
    url = 'http://www.arturocuriel.com',
    download_url = 'https://github.com/elmugrearturo/multilang_summarizer/archive/1.7.tar.gz',
    include_package_data=True,
    package_data={
        'multilang_summarizer': ['languages/*.txt'],
    },
    keywords = ['SUMMARIZATION', 'MULTILANGUAGE', 'RULE-BASED'],
    install_requires=[
            'nltk',
            'pyphen',
            'textstat',
            'sentence-splitter',
            'numpy',
        ],
    classifiers=[
      'Development Status :: 4 - Beta',
      'Intended Audience :: Developers',
      'Topic :: Software Development :: Build Tools',
      'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
      'Programming Language :: Python :: 3',
      'Programming Language :: Python :: 3.4',
      'Programming Language :: Python :: 3.5',
      'Programming Language :: Python :: 3.6',
      'Programming Language :: Python :: 3.7',
    ],
)



import pathlib

import os
from functools import partial
from sentence_splitter import SentenceSplitter
import nltk.data
from nltk.corpus import stopwords
import pyphen

import re
import regex
import string

from collections import OrderedDict

PARENT_DIR = pathlib.Path(__file__).parent

language_index = {"ast" : "Asturian",
                  "bg" : "Bulgarian",
                  "ca" : "Catalan",
                  "cs" : "Czech",
                  "cy" : "Welsh",
                  "de" : "German",
                  "en" : "English",
                  "es" : "Spanish",
                  "et" : "Estonian",
                  "fa" : "Farsi",
                  "fr" : "French",
                  "ga" : "Irish",
                  "gd" : "Scottish Gaelic",
                  "gl" : "Galician",
                  "gv" : "Manx Gaelic",
                  "hu" : "Hungarian",
                  "it" : "Italian",
                  "pt" : "Portuguese",
                  "ro" : "Romanian",
                  "sk" : "Slovak",
                  "sl" : "Slovene",
                  "sv" : "Swedish",
                  "uk" : "Ukrainian"}

nltk_stopwords = {"ar" : 'arabic',
                  "az" : 'azerbaijani',
                  "da" : 'danish',
                  "nl" : 'dutch',
                  "en" : 'english',
                  "fi" : 'finnish',
                  "fr" : 'french',
                  "de" : 'german',
                  "el" : 'greek',
                  "hu" : 'hungarian',
                  "in" : 'indonesian',
                  "it" : 'italian',
                  "kk" : 'kazakh',
                  "ne" : 'nepali',
                  "no" : 'norwegian',
                  "pt" : 'portuguese',
                  "ro" : 'romanian',
                  "ru" : 'russian',
                  "es" : 'spanish',
                  "sv" : 'swedish',
                  "tr" : 'turkish'}

punkt_tokenizers = {"da" : 'danish.pickle',
                    "et" : 'estonian.pickle',
                    "de" : 'german.pickle',
                    "no" : 'norwegian.pickle',
                    "sl" : 'slovene.pickle',
                    "tr" : 'turkish.pickle',
                    "nl" : 'dutch.pickle',
                    "fi" : 'finnish.pickle',
                    "el" : 'greek.pickle',
                    "pl" : 'polish.pickle',
                    "es" : 'spanish.pickle',
                    "cs" : 'czech.pickle',
                    "en" : 'english.pickle',
                    "fr" : 'french.pickle',
                    "it" : 'italian.pickle',
                    "pt" : 'portuguese.pickle',
                    "sv" : 'swedish.pickle'}

splitter_sent_tok = {"ca" : 'Catalan (ca)',
                     "cs" : 'Czech (cs)',
                     "da" : 'Danish (da)',
                     "nl" : 'Dutch (nl)',
                     "en" : 'English (en)',
                     "fi" : 'Finnish (fi)',
                     "fr" : 'French (fr)',
                     "de" : 'German (de)',
                     "el" : 'Greek (el)',
                     "hu" : 'Hungarian (hu)',
                     "is" : 'Icelandic (is)',
                     "it" : 'Italian (it)',
                     "lv" : 'Latvian (lv)',
                     "lt" : 'Lithuanian (lt)',
                     "no" : 'Norwegian (Bokmål) (no)',
                     "pl" : 'Polish (pl)',
                     "pt" : 'Portuguese (pt)',
                     "ro" : 'Romanian (ro)',
                     "ru" : 'Russian (ru)',
                     "sk" : 'Slovak (sk)',
                     "sl" : 'Slovene (sl)',
                     "es" : 'Spanish (es)',
                     "sv" : 'Swedish (sv)',
                     "tr" : 'Turkish (tr)'}

pyphen_dicts = {"de" : "de_DE",
                "en" : "en",
                "es" : "es",
                "fr" : "fr",
                "hu" : "hu_HU",
                "it" : "it",
                "pt" : "pt_BR",
                "ro" : "ro",
                "sv" : "sv"}


class Lemmatizer(object):

    def __init__(self, term_dictionary, language_code, language_name):
        self._term_dictionary = term_dictionary
        self._language_code = language_code
        self._language_name = language_name
        if self._language_code in punkt_tokenizers:
            splitter = nltk.data.load(
                "tokenizers/punkt/%s" % punkt_tokenizers[self._language_code]
                )
            self.sent_split = splitter.tokenize
        elif self._language_code in splitter_sent_tok:
            splitter = SentenceSplitter(language=self._language_code)
            self.sent_split = splitter.split
        else:
            # If nothing works, use naive sentence splitter
            self.sent_split = partial(re.split,
                                r'(?<!\w\.\w.)(?<![A-Z][a-z]\.)(?<=\.|\?)\s')
        self._lemmas = set([lemma for lemma in term_dictionary.values()])
        if self._language_code in nltk_stopwords:
            self._stopwords = stopwords.words(
                                            nltk_stopwords[self._language_code]
                                            )
        else:
            print("No stopwords:", self._language_code)

        # Change punctuation for whitespace
        self.remove_punctuation = partial(regex.sub, '[\p{P}]+', ' ')

        self.pyphen_dic = pyphen.Pyphen(lang=pyphen_dicts[self._language_code])

    def syllabicator(self, term):
        hyphenated = self.pyphen_dic.inserted(term)
        return hyphenated.split("-")

    def tokenize(self, text):
        unclean_tokenized_text = re.sub(r' +', ' ', text).split(" ")
        tokenized_text = [token for token in unclean_tokenized_text if token !=
                         ""]
        return tokenized_text

    def __getitem__(self, key):
        try:
            return self._term_dictionary[key.lower()]
        except:
            if key.lower() in self._lemmas:
                return key.lower()
            return key

    def __len__(self):
        return len(self._term_dictionary)

    def __str__(self):
        return "%s (%s): %d lemmas" % (self._language_name,
                                       self._language_code,
                                       len(self))
    def __repr__(self):
        return str(self)

    def lemmatize(self, paragraph, remove_stopwords=True):

        # Separate sentences
        paragraph = paragraph.strip()
        sentences = self.sent_split(paragraph)

        tok_sentences = []
        lem_sentences = []
        # Separate tokens
        for sentence in sentences:
            tokens = self.tokenize(self.remove_punctuation(sentence))
            # Lemmatize
            lem_tokens = [self[token] for token in tokens]
            # Remove stopwords
            if remove_stopwords:
                lem_tokens = [token for token in lem_tokens if token not in
                              self._stopwords]
            tok_sentences.append(tokens)
            lem_sentences.append(lem_tokens)

        return sentences, tok_sentences, lem_sentences

    @classmethod
    def for_language(klass, language_code, languages_path=PARENT_DIR / "languages/"):
        '''
        ast - Asturian
        bg - Bulgarian
        ca - Catalan
        cs - Czech
        cy - Welsh
        de - German
        en - English
        es - Spanish
        et - Estonian
        fa - Farsi
        fr - French
        ga - Irish
        gd - Scottish Gaelic
        gl - Galician
        gv - Manx Gaelic
        hu - Hungarian
        it - Italian
        pt - Portuguese
        ro - Romanian
        sk - Slovak
        sl - Slovene
        sv - Swedish
        uk - Ukrainian
        '''

        file_path = languages_path / ("lemmatization-%s.txt" % language_code)

        index = OrderedDict({})
        with open(file_path, "r") as f:
            for line in f:
                try:
                    value, key = line.replace("\ufeff", "").strip().split("\t")
                except:
                    print(language_index[language_code])
                    pass
                index[key] = value
        return Lemmatizer(index, language_code, language_index[language_code])
